<?php

namespace EventPortal\Tests\Feature\Admin\Event;

use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Team\Models\Team;
use EventPortal\Tests\Support\Traits\InteractsWithTeams;
use EventPortal\Tests\Support\Traits\SeedRolesAndPermissions;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Testing\TestResponse;
use Spatie\Snapshots\MatchesSnapshots;

class EventEditTest extends TestCase
{
    use RefreshDatabase, MatchesSnapshots, SeedRolesAndPermissions, InteractsWithTeams;

    /** @test **/
    public function can_edit_event()
    {
        $event = factory(Event::class)
            ->states('has-team', 'snapshot')
            ->create();

        $this->assertMatchesJsonSnapshot(
            $this
                ->actingAsSuperAdmin()
                ->get(route('admin.event.edit', $event->id))
                ->props()
        );
    }

    /** @test **/
    public function admins_who_are_in_a_team_can_not_see_other_teams(): void
    {
        $event = factory(Event::class)->state('snapshot')->create();
        factory(Team::class)->create(['name' => 'SHOULD NOT SEE']);
        $adminTeam = factory(Team::class)->create(['name' => 'SHOULD SEE']);
        $admin = factory(Admin::class)->state('admin')->create();

        $this->addToSameTeam([$admin, $event], $adminTeam);

        $this->actingAsAdmin($admin);
        $response = $this->get(route('admin.event.edit', $event->id));

        $this->assertGreaterThanOrEqual(1, $this->countOccurrences($response, 'SHOULD SEE'));
        $this->assertEquals(0, $this->countOccurrences($response, 'SHOULD NOT SEE'));
        $this->assertMatchesJsonSnapshot($response->props());
    }

    /** @test **/
    public function super_admins_who_are_in_a_team_can_see_all_teams(): void
    {
        $event = factory(Event::class)->state('snapshot')->create();
        $team = factory(Team::class)->create(['name' => 'SHOULD SEE']);
        $superAdminTeam = factory(Team::class)->create(['name' => 'SHOULD SEE']);
        $superAdmin = factory(Admin::class)->state('super-admin')->create();

        $this->addToSameTeam([$superAdmin], $superAdminTeam);
        $this->addToSameTeam([$event], $team);

        $this->actingAsSuperAdmin($superAdmin);
        $response = $this->get(route('admin.event.edit', $event->id));

        $this->assertGreaterThanOrEqual(2, $this->countOccurrences($response, 'SHOULD SEE'));
        $this->assertMatchesJsonSnapshot($response->props());
    }

    private function countOccurrences(TestResponse $response, string $needle): int
    {
        return substr_count(json_encode($response->props('form.content')), $needle);
    }
}
